"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.passThrough = passThrough;
const keymap_1 = require("@liqvid/keymap");
/**
 * Handle key sequences in `seqs` even if key capture is suspended.
 * @param keymap {@link Keymap} to handle key sequences.
 * @param seqs Key sequences to handle.
 */
function passThrough(keymap, seqs = []) {
    return seqs.map((key) => {
        const can = cm2lv(key);
        // argh
        const fake = typeof window === "undefined" ? null : new KeyboardEvent("keydown");
        return {
            key,
            run: () => {
                const handlers = keymap.getHandlers(can);
                for (const cb of handlers) {
                    cb(fake);
                }
                return false;
            },
        };
    });
}
/**
 * Convert CodeMirror key sequences to Liqvid format.
 */
function cm2lv(seq) {
    const isMac = typeof globalThis.navigator !== "undefined" &&
        navigator.platform === "MacIntel";
    // biome-ignore lint/style/noParameterAssign: this is fine
    seq = seq.replace("Mod", isMac ? "Meta" : "Ctrl");
    // biome-ignore lint/style/noParameterAssign: this is fine
    seq = seq.replace(/-/g, "+");
    return keymap_1.Keymap.normalize(seq);
}
