"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FakeSelection = void 0;
exports.fakeSelection = fakeSelection;
const state_1 = require("@codemirror/state");
exports.FakeSelection = state_1.StateEffect.define();
/**
 * CodeMirror extension to imitate selections.
 * @param drawSelection CodeMirror extension to modify.
 */
function fakeSelection(drawSelection) {
    const layers = drawSelection.filter((extn) => Array.isArray(extn) && extn.length === 2);
    // @ts-expect-error create is not exposed
    const create = layers[0][0].create;
    return (view) => {
        const instance = create(view);
        instance.cursorLayer.classList.add("fake");
        Object.assign(instance.cursorLayer.style, {
            animationIterationCount: "infinite",
            animationName: "cm-blink",
            animationTimingFunction: "steps(1)",
        });
        // intercept read method
        // this is so evil
        const readPos = instance.measureReq.read;
        instance.measureReq.read = function () {
            const ranges = this.view.state.selection.ranges;
            if (this.range) {
                // @ts-expect-error ranges is readonly
                this.view.state.selection.ranges = [this.range];
            }
            const measure = readPos();
            // @ts-expect-error ranges is readonly
            this.view.state.selection.ranges = ranges;
            for (const c of measure.cursors) {
                const draw = c.draw.bind(c);
                c.draw = () => {
                    const elt = draw();
                    elt.style.display = "block";
                    return elt;
                };
            }
            return measure;
        }.bind(instance);
        // update method
        instance.update = function (update) {
            const effects = update.transactions
                .map((tr) => tr.effects.filter((e) => e.is(exports.FakeSelection)))
                .reduce((a, b) => a.concat(b), []);
            if (effects.length > 0) {
                this.range = state_1.SelectionRange.fromJSON(effects[effects.length - 1].value);
                this.view.requestMeasure(this.measureReq);
            }
        };
        return instance;
    };
}
