"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeRecording = exports.CodeRecorder = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const view_1 = require("@codemirror/view");
const recording_1 = require("@liqvid/recording");
const misc_1 = require("@liqvid/utils/misc");
const icon_1 = require("./icon");
const _1 = require(".");
// the actual thingy that gets exported
class CodeRecorder extends recording_1.ReplayDataRecorder {
    constructor() {
        super();
        (0, misc_1.bind)(this, ["extension"]);
    }
    /**
     * Get a CodeMirror extension for recording.
     * @param specialKeys Map of key sequences to commands, e.g. {"Mod-Enter": "run"}.
     * @returns CodeMirror extension.
     */
    extension(specialKeys = {}) {
        // legacy
        if (Array.isArray(specialKeys)) {
            // biome-ignore lint/style/noParameterAssign: helpful for backwards compatibility
            specialKeys = Object.fromEntries(specialKeys.map((key) => [key, key]));
        }
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        const $this = this;
        const scrollListener = view_1.ViewPlugin.fromClass(class {
            constructor(view) {
                view.scrollDOM.addEventListener("scroll", () => {
                    if (!$this.manager || $this.manager.paused || !$this.manager.active)
                        return;
                    const time = $this.manager.getTime();
                    const fontSize = Number.parseFloat(getComputedStyle(view.scrollDOM).getPropertyValue("font-size"));
                    // vertical scroll is more common so we put it first and omit
                    // horizontal scroll if it's zero
                    const action = [
                        _1.scrollCmd,
                        view.scrollDOM.scrollTop / fontSize,
                        view.scrollDOM.scrollLeft / fontSize,
                    ];
                    if (action[2] === 0) {
                        action.pop();
                    }
                    $this.capture(time, action);
                });
            }
        });
        // record document changes
        const updateListener = view_1.EditorView.updateListener.of((update) => {
            if (!this.manager || this.manager.paused || !this.manager.active)
                return;
            // get selection change (if any)
            const transactions = update.transactions
                .map((t) => {
                if (!t.selection)
                    return null;
                const range = t.selection.ranges[0];
                return [range.anchor, range.head];
            })
                .filter(Boolean)
                .slice(-1);
            // empty events can break replay
            if (update.changes.empty && transactions.length === 0)
                return;
            this.capture(this.manager.getTime(), [
                update.changes.toJSON(),
                ...transactions,
                // biome-ignore lint/suspicious/noExplicitAny: TODO
            ]);
        });
        // record special key presses
        const keyListener = view_1.keymap.of(Object.keys(specialKeys).map((key) => ({
            key,
            run: () => {
                if (this.manager?.active && !this.manager.paused) {
                    this.capture(this.manager.getTime(), specialKeys[key]);
                }
                return false;
            },
        })));
        return [scrollListener, updateListener, keyListener];
    }
}
exports.CodeRecorder = CodeRecorder;
const KeySaveComponent = (props) => {
    return ((0, jsx_runtime_1.jsx)(jsx_runtime_1.Fragment, { children: (0, jsx_runtime_1.jsx)("textarea", { readOnly: true, value: JSON.stringify(props.data) }) }));
};
exports.CodeRecording = {
    enabled: () => true,
    icon: icon_1.icon,
    key: "codemirror",
    name: "Code",
    recorder: new CodeRecorder(),
    saveComponent: KeySaveComponent,
    title: "Record code",
};
